/****** TEV = Test GUI Events = 03.2003 *********************************/
/*
* This program demonstrates usage of TG package:
*   - getting GUI events, dumping GUI events in log-box GUI element
*   - opening a child window
*
* Compilation options - set user input mode by macro:
*   ENABLE_TI_MOUSE = 0 : Use keyboard
*                       : Use mouse (default)
*
* Created: Plamen Kozhuharov, 02.03.2003
*/

#include <string.h>
#include <stdio.h>
#undef stdin
#undef stdout
#undef stderr

#ifdef _ZC_31
  #include <stdlib.h>
  #include <process.h>
  #define  print  printf
  #define  sprint sprintf
#endif

#ifdef _TI_CAMERA
  #include <sysvar.h>
  #include <vcrt.h>       /* Memory allocation    */
  #include <vclib.h>
#endif

#include "tg.h"

/* DDD */
#define ENABLE_TI_MOUSE   1
#define CHANGE_FONT       0

#ifdef _ZC_31
  #define FONT_FILE  "fnt8x16.fnt"
#endif

#ifdef _TI_CAMERA
  #define FONT_FILE  "fd:fnt8x16.dat"
#endif

/* Spin toggle texts */
char *toggle_buf[] = { "Toggle state 0",
                       "Toggle state. 1",
                       "Toggle state.. 2",
                       "Toggle state... 3",
                       "Toggle state.. 4",
                       "Toggle state. 5" };
TG_INT16 toggle_cnt = 6;
TG_INT16 toggle_state = 0;

/* Radio-button texts */
char *radb_txt[] = { "Radio-button 0",
                     "Radio-button . 1",
                     "Radio-button .. 2",
                     "Radio-button ... 3",
                     "Radio-button .... 4",
                     "Radio-button ..... 5" };

char *cbox_txt = "Check-box text";

char *but_msg_press[] = { "Text button event: press",
                          "Mono icon button event: press",
                          "Color icon button event: press" };

char *but_msg_pressed[] = { "Text button event: pressed",
                          "Mono icon button event: pressed",
                          "Color icon button event: pressed" };

char *but_msg_click[] = { "Text button event: click",
                          "Mono icon button event: click",
                          "Color icon button event: click"  };

char *but_msg_focus[] = { "Text button event: on focus",
                          "Mono icon button event: on focus",
                          "Color icon button event: on focus"  };

//****** qsign
#define SIGN_DX 32
#define SIGN_DY 32

#ifdef _ZC_31
  #define W  TG_COLOR_WHITE
  #define B  TG_COLOR_BLUE
  #define G  TG_COLOR_GREEN
  #define R  TG_COLOR_RED
#endif

#ifdef _TI_CAMERA
  #define W  0x3C         /* 15 = TG_COLOR_WHITE */
  #define B  0x04         /*  1 = TG_COLOR_BLUE  */
  #define G  0x08         /*  2 = TG_COLOR_GREEN */
  #define R  0x10         /*  4 = TG_COLOR_RED   */
#endif

        char qsign_buf[] =
{
  W,W,W,W,W,W,W,W,W,W,W,W,0,0,0,0, 0,0,0,0,W,W,W,W,W,W,W,W,W,W,W,W,
  W,W,W,W,W,W,W,W,W,0,0,0,G,G,G,G, G,G,G,G,0,0,0,W,W,W,W,W,W,W,W,W,
  W,W,W,W,W,W,W,0,0,G,G,G,G,G,G,G, G,G,G,G,G,G,G,0,0,W,W,W,W,W,W,W,
  W,W,W,W,W,W,0,G,G,G,G,G,G,W,W,W, W,W,W,W,G,G,G,G,G,0,W,W,W,W,W,W,
  W,W,W,W,W,0,G,G,G,G,G,W,W,W,W,W, W,W,W,W,W,W,G,G,G,G,0,W,W,W,W,W,
  W,W,W,W,0,G,G,G,G,G,W,W,W,W,W,W, W,W,W,W,W,W,W,G,G,G,G,0,W,W,W,W,
  W,W,W,0,G,G,G,G,G,G,W,W,W,W,W,W, W,W,W,W,W,W,W,G,G,G,G,G,0,W,W,W,
  W,W,0,G,G,G,G,G,G,W,W,W,W,W,W,W, W,W,W,W,W,W,W,W,G,G,G,G,G,0,W,W,
  W,W,0,G,G,G,G,G,G,W,W,W,W,W,W,G, G,G,W,W,W,W,W,W,G,G,G,G,G,0,W,W,
  W,0,G,G,G,G,G,G,G,W,W,W,W,W,G,G, G,G,G,W,W,W,W,W,G,G,G,G,G,G,0,W,
  W,0,G,G,G,G,G,G,G,W,W,W,W,W,G,G, G,G,G,W,W,W,W,W,G,G,G,G,G,G,0,W,
  W,0,G,G,G,G,G,G,G,G,W,W,W,W,G,G, G,G,G,W,W,W,W,W,G,G,G,G,G,G,0,W,
  0,G,G,G,G,G,G,G,G,G,G,G,G,G,G,G, G,G,W,W,W,W,W,G,G,G,G,G,G,G,G,0,
  0,G,G,G,G,G,G,G,G,G,G,G,G,G,G,G, G,W,W,W,W,W,W,G,G,G,G,G,G,G,G,0,
  0,G,G,G,G,G,G,G,G,G,G,G,G,G,G,G, W,W,W,W,W,W,G,G,G,G,G,G,G,G,G,0,
  0,G,G,G,G,G,G,G,G,G,G,G,G,G,G,W, W,W,W,W,W,W,G,G,G,G,G,G,G,G,G,0,
  0,G,G,G,G,G,G,G,G,G,G,G,G,G,G,W, W,W,W,W,W,G,G,G,G,G,G,G,G,G,G,0,
  0,G,G,G,G,G,G,G,G,G,G,G,G,G,W,W, W,W,W,W,G,G,G,G,G,G,G,G,G,G,G,0,
  0,G,G,G,G,G,G,G,G,G,G,G,G,G,W,W, W,W,W,W,G,G,G,G,G,G,G,G,G,G,G,0,
  0,G,G,G,G,G,G,G,G,G,G,G,G,G,W,W, W,W,W,G,G,G,G,G,G,G,G,G,G,G,G,0,
  W,0,G,G,G,G,G,G,G,G,G,G,G,G,W,W, W,W,W,G,G,G,G,G,G,G,G,G,G,G,0,W,
  W,0,G,G,G,G,G,G,G,G,G,G,G,G,G,G, G,G,G,G,G,G,G,G,G,G,G,G,G,G,0,W,
  W,0,G,G,G,G,G,G,G,G,G,G,G,G,G,G, G,G,G,G,G,G,G,G,G,G,G,G,G,G,0,W,
  W,W,0,G,G,G,G,G,G,G,G,G,G,G,W,W, W,W,W,G,G,G,G,G,G,G,G,G,G,0,W,W,
  W,W,0,G,G,G,G,G,G,G,G,G,G,W,W,W, W,W,W,W,G,G,G,G,G,G,G,G,G,0,W,W,
  W,W,W,0,G,G,G,G,G,G,G,G,G,W,W,W, W,W,W,W,G,G,G,G,G,G,G,G,0,W,W,W,
  W,W,W,W,0,G,G,G,G,G,G,G,G,W,W,W, W,W,W,W,G,G,G,G,G,G,G,0,W,W,W,W,
  W,W,W,W,W,0,G,G,G,G,G,G,G,G,W,W, W,W,W,G,G,G,G,G,G,G,0,W,W,W,W,W,
  W,W,W,W,W,W,0,G,G,G,G,G,G,G,G,G, G,G,G,G,G,G,G,G,G,0,W,W,W,W,W,W,
  W,W,W,W,W,W,W,0,0,G,G,G,G,G,G,G, G,G,G,G,G,G,G,0,0,W,W,W,W,W,W,W,
  W,W,W,W,W,W,W,W,W,0,0,0,G,G,G,G, G,G,G,G,0,0,0,W,W,W,W,W,W,W,W,W,
  W,W,W,W,W,W,W,W,W,W,W,W,0,0,0,0, 0,0,0,0,W,W,W,W,W,W,W,W,W,W,W,W,
};

//****** wsign
        char wsign_buf[] =
{
  W,W,W,W,W,W,W,W,W,W,W,W,0,0,0,0, 0,0,0,0,W,W,W,W,W,W,W,W,W,W,W,W,
  W,W,W,W,W,W,W,W,W,0,0,0,R,R,R,R, R,R,R,R,0,0,0,W,W,W,W,W,W,W,W,W,
  W,W,W,W,W,W,W,0,0,R,R,R,R,R,R,R, R,R,R,R,R,R,R,0,0,W,W,W,W,W,W,W,
  W,W,W,W,W,W,0,R,R,R,R,R,R,R,W,W, W,W,W,R,R,R,R,R,R,0,W,W,W,W,W,W,
  W,W,W,W,W,0,R,R,R,R,R,R,R,W,W,W, W,W,W,W,R,R,R,R,R,R,0,W,W,W,W,W,
  W,W,W,W,0,R,R,R,R,R,R,R,W,W,W,W, W,W,W,W,W,R,R,R,R,R,R,0,W,W,W,W,
  W,W,W,0,R,R,R,R,R,R,R,W,W,W,W,W, W,W,W,W,W,W,R,R,R,R,R,R,0,W,W,W,
  W,W,0,R,R,R,R,R,R,R,R,W,W,W,W,W, W,W,W,W,W,W,R,R,R,R,R,R,R,0,W,W,
  W,W,0,R,R,R,R,R,R,R,R,W,W,W,W,W, W,W,W,W,W,W,R,R,R,R,R,R,R,0,W,W,
  W,0,R,R,R,R,R,R,R,R,R,W,W,W,W,W, W,W,W,W,W,W,R,R,R,R,R,R,R,R,0,W,
  W,0,R,R,R,R,R,R,R,R,R,R,W,W,W,W, W,W,W,W,W,R,R,R,R,R,R,R,R,R,0,W,
  W,0,R,R,R,R,R,R,R,R,R,R,W,W,W,W, W,W,W,W,W,R,R,R,R,R,R,R,R,R,0,W,
  0,R,R,R,R,R,R,R,R,R,R,R,R,W,W,W, W,W,W,W,R,R,R,R,R,R,R,R,R,R,R,0,
  0,R,R,R,R,R,R,R,R,R,R,R,R,W,W,W, W,W,W,W,R,R,R,R,R,R,R,R,R,R,R,0,
  0,R,R,R,R,R,R,R,R,R,R,R,R,R,W,W, W,W,W,R,R,R,R,R,R,R,R,R,R,R,R,0,
  0,R,R,R,R,R,R,R,R,R,R,R,R,R,W,W, W,W,W,R,R,R,R,R,R,R,R,R,R,R,R,0,
  0,R,R,R,R,R,R,R,R,R,R,R,R,R,W,W, W,W,W,R,R,R,R,R,R,R,R,R,R,R,R,0,
  0,R,R,R,R,R,R,R,R,R,R,R,R,R,W,W, W,W,W,R,R,R,R,R,R,R,R,R,R,R,R,0,
  0,R,R,R,R,R,R,R,R,R,R,R,R,R,W,W, W,W,W,R,R,R,R,R,R,R,R,R,R,R,R,0,
  0,R,R,R,R,R,R,R,R,R,R,R,R,R,R,W, W,W,R,R,R,R,R,R,R,R,R,R,R,R,R,0,
  W,0,R,R,R,R,R,R,R,R,R,R,R,R,R,R, R,R,R,R,R,R,R,R,R,R,R,R,R,R,0,W,
  W,0,R,R,R,R,R,R,R,R,R,R,R,R,R,R, R,R,R,R,R,R,R,R,R,R,R,R,R,R,0,W,
  W,0,R,R,R,R,R,R,R,R,R,R,R,R,W,W, W,W,W,R,R,R,R,R,R,R,R,R,R,R,0,W,
  W,W,0,R,R,R,R,R,R,R,R,R,R,W,W,W, W,W,W,W,R,R,R,R,R,R,R,R,R,0,W,W,
  W,W,0,R,R,R,R,R,R,R,R,R,W,W,W,W, W,W,W,W,W,R,R,R,R,R,R,R,R,0,W,W,
  W,W,W,0,R,R,R,R,R,R,R,R,W,W,W,W, W,W,W,W,W,R,R,R,R,R,R,R,0,W,W,W,
  W,W,W,W,0,R,R,R,R,R,R,R,W,W,W,W, W,W,W,W,W,R,R,R,R,R,R,0,W,W,W,W,
  W,W,W,W,W,0,R,R,R,R,R,R,R,W,W,W, W,W,W,W,R,R,R,R,R,R,0,W,W,W,W,W,
  W,W,W,W,W,W,0,R,R,R,R,R,R,R,W,W, W,W,W,R,R,R,R,R,R,0,W,W,W,W,W,W,
  W,W,W,W,W,W,W,0,0,R,R,R,R,R,R,R, R,R,R,R,R,R,R,0,0,W,W,W,W,W,W,W,
  W,W,W,W,W,W,W,W,W,0,0,0,R,R,R,R, R,R,R,R,0,0,0,W,W,W,W,W,W,W,W,W,
  W,W,W,W,W,W,W,W,W,W,W,W,0,0,0,0, 0,0,0,0,W,W,W,W,W,W,W,W,W,W,W,W,
};

TG_IMAGE qsign_img = { qsign_buf, SIGN_DX, SIGN_DY, SIGN_DX };
TG_IMAGE wsign_img = { wsign_buf, SIGN_DX, SIGN_DY, SIGN_DX };


/*............. Function prototypes */
TG_RET child_win();


/****** main = Main test function = 03.2003 *****************************/
void main()
{
        TG_RET rc;
        TG_BOX   scr_box;

        char    *win_title = "TG demo - Show GUI events";
        TG_COORD win_x,win_y;
        TG_COORD win_dx,win_dy;
        TG_INT16 win_attr;
        TG_INT16 win_depth;
        TG_COORD client_dx,client_dy;

        TG_COORD x,y;
        TG_COORD dx,dy;
        TG_COORD depth;
        TG_COORD x1;

//        TG_COLOR tclr = TG_COLOR_BLUE;
        TG_COLOR tclr = TG_COLOR_WHITE;

/* Radio-button menu parameters */
        TG_UINT16 radb_cnt;
        TG_INT16 radb_val;
        TG_INT16 radb_typ;
        TG_INT16 radb_dist;

/* GUI identifiers */
        TG_INT16 win_id;
        TG_INT16 but_id;
        TG_INT16 but1_id;
        TG_INT16 but2_id;
        TG_INT16 edit_id;
        TG_INT16 spin1_id;
        TG_INT16 spin2_id;
        TG_INT16 spin3_id;
        TG_INT16 radb_id;
        TG_INT16 cbox_id;
        TG_INT16 text_id;
        TG_INT16 logb_id;

        TG_INT16 gui_id;       /* GUI identifier        */
        TG_EVENT gui_event;    /* GUI event             */

/* GUI data */
        TG_INT32   int_val;
        double  float_val;
        char    txt_val[80];

        char     str[160];
        TG_INT16 font_dx,font_dy;
        char    *ptr;
        int      i;


//............. Init TG package
        rc = tg_init();
        if(rc != RC_OK)
        {
           print("TG init error = %d\n",rc);
           return;
        }

/*............. Set input mode */
/* Note: The function tg_set_inputmode() initializes mouse ! */
#ifdef _ZC_31
        rc = tg_set_inputmode(TG_INPUT_MODE_MOUSE | TG_INPUT_MODE_KBD);
        if(rc != RC_OK) goto done;
#endif

#ifdef _TI_CAMERA
  #if ENABLE_TI_MOUSE
        rc = tg_set_inputmode(TG_INPUT_MODE_MOUSE);
  #else
        rc = tg_set_inputmode(TG_INPUT_MODE_KBD);
  #endif
        if(rc != RC_OK) goto done;
#endif

//............. Set new font
#if CHANGE_FONT
        rc = tg_set_font(FONT_FILE);
//        if(rc != RC_OK) goto done;
#endif

//............. Clear screen
        tg_get_screenbox(scr_box);
        rc = tg_fill_box(scr_box,TG_COLOR_CLEAR,TG_MODE_SET);
        if(rc != RC_OK) goto done;

//............. Get font size
        tg_get_fontsize(&font_dx,&font_dy);

/*----------------------------------------------------------------------*/
/*                               Open window                            */
/*----------------------------------------------------------------------*/
        win_x = 10;
        win_y = 10;
        win_dx = 620;
        win_dy = 400;
        win_attr = TG_WIN_CLOSE_BUT;
        win_depth = 2;

        rc = tg_WinOpen(win_title,win_x,win_y,win_dx,win_dy,win_attr,
                        win_depth,
                        &win_id);
        if(rc != RC_OK) goto done;

/*............. Get size of client area box */
        rc = tg_WinClientSize(&client_dx,&client_dy);
        if(rc != RC_OK) goto done;

/*----------------------------------------------------------------------*/
/*                          Open window controls                        */
/*----------------------------------------------------------------------*/
/* Note: GUI element coordinates are relative to window client area !   */

#define SKIP_DIST   10
#define SKIP_DIST1  2

/*............. Open text button */
        x1 = 5;
        x = x1 + 156;

        y = 20;
        dx = 100;
        dy = 24;
        depth = 2;

//        rc = tg_disp_text("Text button:",x1,y,tclr,TG_COLOR_DEF_BGND);
        rc = tg_TextOpen("Text button:",x1,y,0,
                          tclr,TG_COLOR_CLEAR,&text_id);
        if(rc != RC_OK) goto done;

        rc = tg_TxtButOpen("Button",
//                         x,y,dx,dy,depth,
                         x,y,dx,dy,1,
                         TG_ALIGN_CENTER,
                         NULL,           // colors
                         &but_id);
        if(rc != RC_OK) goto done;

/*............. Open monochrome icon button */
        y += dy + SKIP_DIST;
        dx = SIGN_DX + 2*depth + 8;
        dy = SIGN_DY + 2*depth + 8;

/* make monochrome picture */
        ptr = qsign_buf;
        for(i=0; i<SIGN_DX*SIGN_DY; i++)
        {
           if(*ptr == W) *ptr = 0;
           ptr++;
        }

//        rc = tg_disp_text("Icon buttons:",x1,y,tclr,TG_COLOR_DEF_BGND);
        rc = tg_TextOpen("Icon buttons:",x1,y,0,
                          tclr,TG_COLOR_CLEAR,&text_id);
        if(rc != RC_OK) goto done;

        rc = tg_IconButOpen(&qsign_img,
                         TG_BUT_TYPE_ICON,
//                         x,y,dx,dy,depth,
                         x,y,dx,dy,1,
                         TG_ALIGN_CENTER,
                         NULL,           // colors
                         &but1_id);
        if(rc != RC_OK) goto done;

/*............. Open color icon button */
        rc = tg_IconButOpen(&wsign_img,
                         TG_BUT_TYPE_CLR_ICON,
                         x+dx+8,y,dx,dy,depth,
                         TG_ALIGN_CENTER,
                         NULL,           // colors
                         &but2_id);
        if(rc != RC_OK) goto done;


/*............. Open edit control */
        y += dy + SKIP_DIST;
        dx = 120;
        dy = 18;

//        rc = tg_disp_text("Edit:",x1,y,tclr,TG_COLOR_DEF_BGND);
        rc = tg_TextOpen("Edit:",x1,y,0,
                          tclr,TG_COLOR_CLEAR,&text_id);
        if(rc != RC_OK) goto done;

        rc = tg_EditOpen("Edit text ...",48,
                         x,y,dx,dy,depth,
                         NULL,           // colors
                         &edit_id);
        if(rc != RC_OK) goto done;

/*............. Open integer spin control */
        y += dy + SKIP_DIST;
        dx = 160;
        dy = 20;

//        rc = tg_disp_text("Spin1 (integer):",x1,y,tclr,TG_COLOR_DEF_BGND);
        rc = tg_TextOpen("Spin1 (integer):",x1,y,0,
                          tclr,TG_COLOR_CLEAR,&text_id);
        if(rc != RC_OK) goto done;

        rc = tg_IntSpinOpen(500,1,-10000,20000,TG_SPIN_EDIT_ON,9,
                         x,y,dx,dy,depth,
                         NULL,           // colors
                         &spin1_id);
        if(rc != RC_OK) goto done;

/*............. Open float spin control */
        y += dy + SKIP_DIST;

        rc = tg_TextOpen("Spin2 (float):",x1,y,0,
                          tclr,TG_COLOR_CLEAR,&text_id);
        if(rc != RC_OK) goto done;

        rc = tg_FloatSpinOpen(1.123,1.0,-1000.0,2000.0,TG_SPIN_EDIT_ON,24,
                         x,y,dx,dy,depth,
                         NULL,           // colors
                         &spin2_id);
        if(rc != RC_OK) goto done;

/*............. Open toggle spin control */
        y += dy + SKIP_DIST;

//        rc = tg_disp_text("Spin3 (toggle):",x1,y,tclr,TG_COLOR_DEF_BGND);
        rc = tg_TextOpen("Spin3 (toggle):",x1,y,0,
                          tclr,TG_COLOR_CLEAR,&text_id);
        if(rc != RC_OK) goto done;

        rc = tg_ToggleSpinOpen(toggle_buf,toggle_cnt,toggle_state,
                         x,y,dx,dy,depth,
                         NULL,           // colors
                         &spin3_id);
        if(rc != RC_OK) goto done;

/*............. Open radio-button menu */
        y += dy + SKIP_DIST + 4;

        radb_cnt = 3;
        radb_val = 0;
        radb_typ = TG_RADIOBUT_TYPE_VERT;       /* vertical menu */
        radb_dist = 4;

        rc = tg_TextOpen("Radio-button menu:",x1,y,0,
                          tclr,TG_COLOR_CLEAR,&text_id);
        if(rc != RC_OK) goto done;

        rc = tg_RadioButOpen(radb_txt,radb_cnt,radb_val,radb_typ,radb_dist,
                         x,y,
                         NULL,
                         &radb_id);
        if(rc != RC_OK) goto done;

/*............. Open check-box */
        dy = radb_cnt*(radb_dist+max(font_dy,12)) - radb_dist;
        y += dy + SKIP_DIST;

//        rc = tg_disp_text("Check-box:",x1,y,tclr,TG_COLOR_DEF_BGND);
        rc = tg_TextOpen("Check-box:",x1,y,0,
                          tclr,TG_COLOR_CLEAR,&text_id);
        if(rc != RC_OK) goto done;

        rc = tg_CheckBoxOpen(cbox_txt,0,x,y,NULL,&cbox_id);
        if(rc != RC_OK) goto done;


/*............. Open event log box (passive GUI element) */
        dx = 280;
        x = client_dx - dx - 10;
        y = 10;

//        rc = tg_disp_text("GUI events:",x,y,tclr,TG_COLOR_DEF_BGND);
        rc = tg_TextOpen("GUI events:",x,y,0,
                          tclr,TG_COLOR_CLEAR,&text_id);
        if(rc != RC_OK) goto done;

        y += font_dy + 2;
        rc = tg_LogBoxOpen(x,y,dx,client_dy - 40,2,NULL,&logb_id);
        if(rc != RC_OK) goto done;


/*----------------------------------------------------------------------*/
/*                             Get GUI events                           */
/*----------------------------------------------------------------------*/

/*............. Get event loop */
        for(;;)
        {
           rc = tg_GetEvent(&gui_id,&gui_event);
           if(rc != RC_OK) goto done;

/*............. No GUI event */
           if(gui_event == TG_GUI_EVENT_NONE) continue;

/*............. Window events */
           if(gui_id == win_id)
           {
              if(gui_event == TG_WIN_EVENT_CLOSE)
              {
                 rc = tg_WinClose();
                 if(rc != RC_OK) goto done;
                 break;
              }
           }

/*............. Button events */
           if(gui_id == but_id || gui_id == but1_id || gui_id == but2_id)
           {
             i = 0;
             if(gui_id == but1_id) i = 1;
             if(gui_id == but2_id) i = 2;

              switch(gui_event)
              {
                 case TG_BUT_EVENT_PRESS:
                    rc = tg_SetGuiData(logb_id,but_msg_press[i]);
                    if(rc != RC_OK) goto done;
                    break;

                 case TG_BUT_EVENT_PRESSED:
                    rc = tg_SetGuiData(logb_id,but_msg_pressed[i]);
                    if(rc != RC_OK) goto done;
                    break;

                 case TG_BUT_EVENT_CLICK:
                    rc = tg_SetGuiData(logb_id,but_msg_click[i]);
                    if(rc != RC_OK) goto done;

/* Open child window when color button is clicked */
                    if(gui_id == but2_id)
                    {
                       rc = child_win();
                       if(rc != RC_OK) goto done;
                    }
                    break;

                 case TG_BUT_EVENT_ONFOCUS:
                    rc = tg_SetGuiData(logb_id,but_msg_focus[i]);
                    if(rc != RC_OK) goto done;
                    break;

              } /* switch(gui_event) */

              continue;

           } /* if(gui_id == but_id) */

/*............. Edit events */
           if(gui_id == edit_id)
           {
              switch(gui_event)
              {
                 case TG_EDIT_EVENT_ONFOCUS:
                    rc = tg_SetGuiData(logb_id,"Edit event: on focus");
                    if(rc != RC_OK) goto done;
                    break;

                 case TG_EDIT_EVENT_CHANGE:
                    rc = tg_SetGuiData(logb_id,"Edit event: changed");
                    if(rc != RC_OK) goto done;

                    rc = tg_GetTextData(edit_id,txt_val,sizeof(str));
                    if(rc != RC_OK) goto done;

                    strcpy(str,"     Text=");
                    strcat(str,txt_val);

                    rc = tg_SetGuiData(logb_id,str);
                    if(rc != RC_OK) goto done;

                    break;

              } /* switch(gui_event) */

              continue;

           } /* if(gui_id == edit_id) */

/*............. Spin 1 (int) events */
           if(gui_id == spin1_id)
           {
              switch(gui_event)
              {
                 case TG_SPIN_EVENT_ONFOCUS:
                    rc = tg_SetGuiData(logb_id,"Spin1 event: on focus");
                    if(rc != RC_OK) goto done;
                    break;

                 case TG_SPIN_EVENT_CHANGE:
                    rc = tg_SetGuiData(logb_id,"Spin1 event: changed");
                    if(rc != RC_OK) goto done;

                    rc = tg_GetIntData(spin1_id,&int_val);
                    if(rc != RC_OK) goto done;

                    sprintf(str,"      Integer value = %d",int_val);

                    rc = tg_SetGuiData(logb_id,str);
                    if(rc != RC_OK) goto done;

                    break;

              } /* switch(gui_event) */

              continue;

           } /* if(gui_id == spin1_id) */

/*............. Spin 2 (float) events */
           if(gui_id == spin2_id)
           {
              switch(gui_event)
              {
                 case TG_SPIN_EVENT_ONFOCUS:
                    rc = tg_SetGuiData(logb_id,"Spin2 event: on focus");
                    if(rc != RC_OK) goto done;
                    break;

                 case TG_SPIN_EVENT_CHANGE:
                    rc = tg_SetGuiData(logb_id,"Spin2 event: changed");
                    if(rc != RC_OK) goto done;

                    rc = tg_GetFloatData(spin2_id,&float_val);
                    if(rc != RC_OK) goto done;

                    sprintf(str,"      Float value = %f",float_val);

                    rc = tg_SetGuiData(logb_id,str);
                    if(rc != RC_OK) goto done;

                    break;

              } /* switch(gui_event) */

              continue;

           } /* if(gui_id == spin2_id) */

/*............. Spin 3 (toggle) events */
           if(gui_id == spin3_id)
           {
              switch(gui_event)
              {
                 case TG_SPIN_EVENT_ONFOCUS:
                    rc = tg_SetGuiData(logb_id,"Spin3 event: on focus");
                    if(rc != RC_OK) goto done;
                    break;

                 case TG_SPIN_EVENT_CHANGE:
                    rc = tg_SetGuiData(logb_id,"Spin3 event: changed");
                    if(rc != RC_OK) goto done;

                    rc = tg_GetIntData(spin3_id,&int_val);
                    if(rc != RC_OK) goto done;

                    sprintf(str,"      Toggle value = %d",int_val);

                    rc = tg_SetGuiData(logb_id,str);
                    if(rc != RC_OK) goto done;

                    break;

              } /* switch(gui_event) */

              continue;

           } /* if(gui_id == spin3_id) */

/*............. Radio-button events */
           if(gui_id == radb_id)
           {
              switch(gui_event)
              {
                 case TG_RADIOBUT_EVENT_ONFOCUS:
                    rc = tg_SetGuiData(logb_id,"Radio-button event: on focus");
                    if(rc != RC_OK) goto done;
                    break;

                 case TG_RADIOBUT_EVENT_CHANGE:
                    rc = tg_SetGuiData(logb_id,"Radio-button event: changed");
                    if(rc != RC_OK) goto done;

                    rc = tg_GetIntData(radb_id,&int_val);
                    if(rc != RC_OK) goto done;

                    sprintf(str,"      Radio-button value = %d",int_val);

                    rc = tg_SetGuiData(logb_id,str);
                    if(rc != RC_OK) goto done;

                    break;

              } /* switch(gui_event) */

              continue;

           } /* if(gui_id == radb_id) */

/*............. Check-box events */
           if(gui_id == cbox_id)
           {
              switch(gui_event)
              {
                 case TG_CHECKBOX_EVENT_ONFOCUS:
                    rc = tg_SetGuiData(logb_id,"Check-box event: on focus");
                    if(rc != RC_OK) goto done;
                    break;

                 case TG_CHECKBOX_EVENT_CHANGE:
                    rc = tg_SetGuiData(logb_id,"Check-box event: changed");
                    if(rc != RC_OK) goto done;

                    rc = tg_GetIntData(cbox_id,&int_val);
                    if(rc != RC_OK) goto done;

                    sprintf(str,"      Check-box value = %d",int_val);

                    rc = tg_SetGuiData(logb_id,str);
                    if(rc != RC_OK) goto done;

                    break;

              } /* switch(gui_event) */

              continue;

           } /* if(gui_id == cbox_id) */

        } /* for(;;) : get event loop */


/*............. Exit */
done:
        tg_exit();
        if(rc != RC_OK)
           print("RC = %d\n",rc);
        return;
}

/****** child_win = Child window = 03.2003 ******************************/
/*
* Open child window.
*/
TG_RET child_win()
{
        TG_RET   rc;
        TG_INT16 win_id;
        TG_INT16 text_id;
        TG_INT16 gui_id;       /* GUI identifier        */
        TG_EVENT gui_event;    /* GUI event             */

/*............. Open child window */
        rc = tg_WinOpen("Child window",280,130,300,100,TG_WIN_CLOSE_BUT,
                        2,
                        &win_id);
        if(rc != RC_OK) goto done;

/*............. Show text */
        rc = tg_TextOpen("Color icon button click !",46,30,0,
                          TG_COLOR_BLACK,TG_COLOR_CLEAR,&text_id);
        if(rc != RC_OK) goto done;

/*............. Get event loop */
        for(;;)
        {
           rc = tg_GetEvent(&gui_id,&gui_event);
           if(rc != RC_OK) goto done;

/*............. Window events */
           if(gui_id == win_id)
           {
              if(gui_event == TG_WIN_EVENT_CLOSE)
              {
                 rc = tg_WinClose();
                 if(rc != RC_OK) goto done;
                 break;
              }
           }
        } /* for(;;) : get event loop */

done:
        return rc;

}
