/****** TGBIN = TG package - binarization demo program = 03.2003 ********/
/*
* This program demonstrates usage of TG package on TI camera. It displays
* a window for setting binarisation parameters and a binarisation box where
* the binarized picture is displayed. The binarisation box can be dragged
* by pressed left mouse button when the mouse cursor is not inside the
* window.
*
* Usage on TI camera:
* The program first attempts to find TGTERM mouse (enabled by Alt-M before
* starting "tgbin"). If mouse is detected, the demo works with TGTERM mouse.
* If mouse is not detected, it works with TGTERM keyboard.
*
* Warning: Use 9600 or higher baud rate when working with TGTERM keyboard.
*
* Supported Esc sequences for non-ASCII keys in TGTERM:
*          CursorUp       Esc [ A
*          CursorDown     Esc [ B
*          CursorRight    Esc [ C
*          CursorLeft     Esc [ D
*          Ins            Esc [ I
*          Home           Esc [ H
*          End            Esc [ K
*          F1             Esc O q
*          F2             Esc O r
*          F3             Esc O s
*          F4             Esc O t
*          F5             Esc O u
*          F6             Esc O v
*          F7             Esc O w
*          F8             Esc O x
*          F9             Esc O y
*          F10            Esc O p
*
* Created: Plamen Kozhuharov, 12.03.2003
*/

#include <string.h>
//#include <stdio.h>
//#undef stdin
//#undef stdout
//#undef stderr

#ifdef _ZC_31
  #include <stdlib.h>
  #define  print  printf
  #define  sprint sprintf
#endif

#ifdef _TI_CAMERA
//  #include <vclib.h>
#endif

#include "tg.h"

/* DDD */
#define CHANGE_FONT   0         /* 1: change default font       */

#ifdef _TI_CAMERA
  #define FONT_FILE  "fd:fnt8x16.dat"
#else
  #define FONT_FILE  "fnt8x16.fnt"
#endif

/*............. Global data */
        TG_BOX      win_box;
        TG_INT32    bin_x,bin_y;
        TG_INT32    bin_dx = 200;
        TG_INT32    bin_dy = 100;

//        TG_COLOR    bin_box_clr = TG_COLOR_WHITE;
        TG_COLOR    bin_box_clr = TG_COLOR_BRIGHT_CYAN;
        TG_BOX      bin_box;

        TG_INT32    bin_th = 128;
        TG_INT32    bin_clr_num = 0;
        int         bin_fill = 0;

        TG_INT32    mark_area = 0;  /* 0=dark, 1=light  */

/* Binarization colors */
        char *bin_clr_txt[] = { "Red",
                                "Green",
                                "Blue",
                                "White",
                                "Black",
                                "Tr. red",
                                "Tr. green",
                                "Tr. blue",
                              };

        TG_COLOR bin_clrs[] = { TG_COLOR_BRIGHT_RED,
                                TG_COLOR_BRIGHT_GREEN,
                                TG_COLOR_BRIGHT_BLUE,
                                TG_COLOR_WHITE,
                                TG_COLOR_BLACK,
                                TG_COLOR_TRANSL_RED,
                                TG_COLOR_TRANSL_GREEN,
                                TG_COLOR_TRANSL_BLUE,
                              };

#define BIN_COLOR_CNT (sizeof(bin_clr_txt)/sizeof(char *))

        char *radb_menu_txt[] = { "Mark dark area",
                                  "Mark light area" };

/*............. Button color icon pictures */
#define BUT_ICON_DX  66
#define BUT_ICON_DY  52

/* Live button picture */
        static char live_but_pict[] =
{
   7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7,
   7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7,14, 7,14, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7,
   7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7,
   7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7,14, 8,14, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7,
   7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7,14, 7, 7, 7, 7, 7, 7, 7, 7, 8, 7, 7, 7, 7, 7, 7, 7, 7, 7,14, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7,
   7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7,14, 7, 7, 7, 7, 7, 0, 7, 7,14, 0,14, 7, 7, 0, 7, 7, 7, 7, 7, 7,14, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7,
   7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 8,14, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7,14, 8, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7,
   7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7,14, 0, 7, 7, 7, 7, 0, 0, 0, 0, 0, 0, 7, 7, 7, 7, 0,14, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7,
   7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 0, 0,14,14,14,14,14,14, 0, 0, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7,
   7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 0,14,14,14,14,14,14,14,14,14,14, 0, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7,
   7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 0, 7, 7, 0,14,14,14,14,14, 0, 0,14,14,14,14,14, 0, 7, 7, 0, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7,
   7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 0,14,14,14,14,14,14, 0, 0,14,14,14,14,14,14, 0, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7,
   7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7,14, 7,14, 7,14, 7, 0,14,14,14,14,14,14, 0, 0,14,14,14,14,14,14, 0, 7,14, 7,14, 7,14, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7,
   7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 8, 0, 7, 0,14,14,14,14,14,14, 0, 0,14,14,14,14,14,14, 0, 7, 0, 8, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7,
   7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7,14, 7,14, 7,14, 7, 0,14,14,14,14,14,14, 0, 0,14,14,14,14,14,14, 0, 7,14, 7,14, 7,14, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7,
   7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 0,14,14,14,14,14,14, 0, 0,14,14,14,14,14,14, 0, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7,
   7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 0, 7, 7, 0,14,14,14,14,14, 0, 0,14,14,14,14,14, 0, 7, 7, 0, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7,
   7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 0,14,14,14,14,14,14,14,14,14,14,14,14, 0, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7,
   7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 0,15, 0,14,14,14,14, 0, 0,14,14,14,14, 0, 0, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7,
   7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 0,15,15,15, 0,14,14,14, 0, 0,14,14,14, 0,15,15, 0, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7,
   7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 0,15,11,15,11,15, 0,14,14,14,14,14,14, 0,11,15,11,15, 0, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7,
   7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 0,15,15,15,15,15,15,15, 0,14,14,14,14, 0,15,15,15,15,15,15, 0, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7,
   7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 0,15,11,15,11,15,11,15,11, 0, 8, 8, 8, 8, 0,15,11,15,11,15,11,15, 0, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7,
   7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 0,15,15,15,15,15,15,15, 0, 7, 7, 7, 7, 0,15,15,15,15,15,15, 0, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7,
   7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 0,15,11,15,11,15,11, 0, 8, 8, 8, 8, 0,15,11,15,11,15, 0, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7,
   7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 0,15,15,15,15,15, 0, 0, 7, 7, 0, 0,15,15,15,15, 0, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7,
   7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 0,15,11,15,11,15, 0, 0, 0, 0,15,15,11,15, 0, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7,
   7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 0,15,15,15,15,15,15,15,15,15,15,15, 0, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7,
   7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 0,15,11,15,11,15,11,15,11,15, 0, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7,
   7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 0,15,15,15,15,15,15,15, 0, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7,
   7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 0,15,11,15,11,15, 0, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7,
   7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 0,15,15,15, 0, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7,
   7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 0, 0, 0, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7,
   7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7,
   7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7,
   7, 0, 0, 7, 7, 7, 7, 7, 7, 7, 0, 0, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 0, 0, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 0, 0, 7, 7, 7,
   7, 0, 0, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 0, 0, 7, 7, 7,
   7, 0, 0, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 0, 0, 7, 7, 7,
   7, 0, 0, 7, 7, 7, 7, 7, 7, 7, 0, 0, 7, 0, 0, 7, 7, 7, 7, 0, 0, 7, 7, 7, 0, 0, 0, 0, 7, 7, 7, 7, 7, 7, 7, 7, 7, 0, 0, 0, 0, 0, 0, 7, 7, 7, 7, 0, 0, 7, 7, 7, 7, 0, 0, 0, 0, 7, 7, 0, 0, 0, 0, 0, 0, 7,
   7, 0, 0, 7, 7, 7, 7, 7, 7, 7, 0, 0, 7, 0, 0, 7, 7, 7, 7, 0, 0, 7, 7, 0, 0, 7, 7, 0, 0, 7, 7, 7, 7, 7, 7, 7, 7, 0, 0, 0, 7, 7, 0, 0, 7, 7, 7, 0, 0, 7, 7, 7, 0, 0, 7, 7, 0, 0, 7, 7, 7, 0, 0, 7, 7, 7,
   7, 0, 0, 7, 7, 7, 7, 7, 7, 7, 0, 0, 7, 7, 0, 0, 7, 7, 0, 0, 7, 7, 0, 0, 7, 7, 7, 7, 0, 0, 7, 7, 7, 7, 7, 7, 7, 0, 0, 7, 7, 7, 7, 0, 0, 7, 7, 0, 0, 7, 7, 0, 0, 7, 7, 7, 7, 0, 0, 7, 7, 0, 0, 7, 7, 7,
   7, 0, 0, 7, 7, 7, 7, 7, 7, 7, 0, 0, 7, 7, 0, 0, 7, 7, 0, 0, 7, 7, 0, 0, 7, 7, 7, 7, 0, 0, 7, 7, 7, 7, 7, 7, 7, 0, 0, 7, 7, 7, 7, 0, 0, 7, 7, 0, 0, 7, 7, 0, 0, 7, 7, 7, 7, 7, 7, 7, 7, 0, 0, 7, 7, 7,
   7, 0, 0, 7, 7, 7, 7, 7, 7, 7, 0, 0, 7, 7, 7, 0, 0, 0, 0, 7, 7, 7, 0, 0, 0, 0, 0, 0, 0, 0, 7, 7, 7, 7, 7, 7, 7, 0, 0, 7, 7, 7, 7, 0, 0, 7, 7, 0, 0, 7, 7, 0, 0, 7, 7, 7, 7, 7, 7, 7, 7, 0, 0, 7, 7, 7,
   7, 0, 0, 7, 7, 7, 7, 7, 7, 7, 0, 0, 7, 7, 7, 0, 0, 0, 0, 7, 7, 7, 0, 0, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 0, 0, 7, 7, 7, 7, 0, 0, 7, 7, 0, 0, 7, 7, 0, 0, 7, 7, 7, 7, 7, 7, 7, 7, 0, 0, 7, 7, 7,
   7, 0, 0, 7, 7, 7, 7, 7, 7, 7, 0, 0, 7, 7, 7, 7, 0, 0, 7, 7, 7, 7, 0, 0, 7, 7, 7, 7, 0, 0, 7, 7, 7, 7, 7, 7, 7, 0, 0, 7, 7, 7, 7, 0, 0, 7, 7, 0, 0, 7, 7, 0, 0, 7, 7, 7, 7, 0, 0, 7, 7, 0, 0, 7, 7, 7,
   7, 0, 0, 7, 7, 7, 7, 7, 7, 7, 0, 0, 7, 7, 7, 7, 0, 0, 7, 7, 7, 7, 7, 0, 0, 7, 7, 0, 0, 7, 7, 7, 7, 7, 7, 7, 7, 0, 0, 0, 7, 7, 0, 0, 7, 7, 7, 0, 0, 7, 7, 7, 0, 0, 7, 7, 0, 0, 7, 7, 7, 0, 0, 7, 7, 7,
   7, 0, 0, 0, 0, 0, 0, 0, 0, 7, 0, 0, 7, 7, 7, 7, 0, 0, 7, 7, 7, 7, 7, 7, 0, 0, 0, 0, 7, 7, 7, 7, 7, 7, 7, 7, 7, 0, 0, 0, 0, 0, 0, 7, 7, 7, 7, 0, 0, 7, 7, 7, 7, 0, 0, 0, 0, 7, 7, 7, 7, 7, 0, 0, 7, 7,
   7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 0, 0, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7,
   7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 0, 0, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7,
   7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 0, 0, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7,
   7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 0, 0, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7,
   7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7,
};

/* Still button picture */
        static char still_but_pict[] =
{
   7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7,
   7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 0, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7,
   7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 8,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15, 8, 0, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7,
   7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 8,15, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 8, 0, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7,
   7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 8,15, 7, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8,15, 7, 8, 0, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7,
   7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 8,15, 7, 8, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,15, 7, 8, 0, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7,
   7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 8,15, 7, 8, 0, 8, 8, 8, 8, 8, 8, 8, 8, 6, 0, 8, 8, 8, 8, 8, 8, 8, 8,15, 7, 8, 0, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7,
   7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 8,15, 7, 8, 0, 8, 9, 9, 9, 9, 9, 9, 9, 6, 0, 9, 9, 9, 9, 9, 9, 9, 8,15, 7, 8, 0, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7,
   7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 8,15, 7, 8, 0, 8, 9, 9, 9, 9, 9, 9, 9, 6, 0, 9, 9,15, 9, 9,15,15, 8,15, 7, 8, 0, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7,
   7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 8,15, 7, 8, 0, 8, 9, 9, 9, 9, 9, 9, 9, 6, 0, 9,15,15,15,15,15,15, 8,15, 7, 8, 0, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7,
   7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 8,15, 7, 8, 0, 8, 9, 9, 9, 9, 9,15,15, 6, 0,15,15,15, 7, 9,15,15, 8,15, 7, 8, 0, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7,
   7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 8,15, 7, 8, 0, 8, 9, 9,15, 9,15,15,15, 6, 0,15, 7, 7,15,15,15,15, 8,15, 7, 8, 0, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7,
   7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 8,15, 7, 8, 0, 8, 9, 9, 9, 9, 9, 9, 7, 6, 0, 9,15, 8, 8, 8,15,15, 8,15, 7, 8, 0, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7,
   7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 8,15, 7, 8, 0, 8, 9, 9, 9, 9, 9, 9, 9, 6, 0, 9, 9, 9, 9, 8,15,15, 8,15, 7, 8, 0, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7,
   7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 8,15, 7, 8, 0, 8, 9, 9, 9, 9, 9, 9, 9, 6, 0, 9, 9, 9, 9, 9, 9, 9, 8,15, 7, 8, 0, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7,
   7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 8,15, 7, 8, 0, 8, 9, 9, 9, 9, 9, 9, 9, 6, 0, 9, 9, 9, 9, 9, 9, 9, 8,15, 7, 8, 0, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7,
   7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 8,15, 7, 8, 0, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 8,15, 7, 8, 0, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7,
   7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 8,15, 7, 8, 0, 0, 0, 0, 0, 0, 0, 0, 0, 6, 0, 0, 0, 0, 0, 0, 0, 0, 8,15, 7, 8, 0, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7,
   7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 8,15, 7, 8, 0, 8,15, 9, 9, 9, 9, 9, 9, 6, 0, 9, 9, 9, 9, 9, 9, 9, 8,15, 7, 8, 0, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7,
   7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 8,15, 7, 8, 0, 8,15,15, 9, 9, 9, 9, 9, 6, 0, 9, 9, 9, 9, 9, 9, 9, 8,15, 7, 8, 0, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7,
   7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 8,15, 7, 8, 0, 8,15,15,15,15, 9, 9, 9, 6, 0, 9, 9, 9, 9, 9, 9, 9, 8,15, 7, 8, 0, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7,
   7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 8,15, 7, 8, 0, 8,15,15, 7, 9,15,15, 9, 6, 0, 9, 9, 9, 9, 9, 9, 9, 8,15, 7, 8, 0, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7,
   7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 8,15, 7, 8, 0, 8, 7,15, 7,15, 7, 7,15, 6, 0,15, 9, 9, 9, 9, 9, 9, 8,15, 7, 8, 0, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7,
   7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 8,15, 7, 8, 0, 8,15, 8, 8, 7, 7, 9, 9, 6, 0, 9, 9, 9, 9, 9, 9, 9, 8,15, 7, 8, 0, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7,
   7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 8,15, 7, 8, 0, 8,15, 9, 9, 9, 9, 9, 9, 6, 0, 9, 9, 9, 9, 9, 9, 9, 8,15, 7, 8, 0, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7,
   7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 8,15, 7, 8, 0, 8, 9, 9, 9, 9, 9, 9, 9, 6, 0, 9, 9, 9, 9, 9, 9, 9, 8,15, 7, 8, 0, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7,
   7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 8,15, 7, 8, 0, 8, 9, 9, 9, 9, 9, 9, 9, 6, 0, 9, 9, 9, 9, 9, 9, 9, 8,15, 7, 8, 0, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7,
   7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 8,15, 7, 8, 0, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8,15, 7, 8, 0, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7,
   7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 8,15, 7, 8,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15, 7, 8, 0, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7,
   7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 8,15, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 8, 0, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7,
   7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 0, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7,
   7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7,
   7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7,
   7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7,
   7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7,
   7, 7, 7, 0, 0, 0, 0, 0, 0, 7, 7, 7, 7, 7, 0, 0, 7, 7, 7, 0, 0, 7, 7, 0, 0, 7, 7, 0, 0, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 0, 0, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 0, 0, 7, 7, 7, 7,
   7, 7, 0, 0, 7, 7, 7, 7, 0, 0, 7, 7, 7, 7, 0, 0, 7, 7, 7, 7, 7, 7, 7, 0, 0, 7, 7, 0, 0, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 0, 0, 7, 7, 7, 7,
   7, 0, 0, 7, 7, 7, 7, 7, 7, 0, 0, 7, 7, 7, 0, 0, 7, 7, 7, 7, 7, 7, 7, 0, 0, 7, 7, 0, 0, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 0, 0, 7, 7, 7, 7,
   7, 0, 0, 7, 7, 7, 7, 7, 7, 7, 7, 7, 0, 0, 0, 0, 0, 0, 7, 0, 0, 7, 7, 0, 0, 7, 7, 0, 0, 7, 7, 7, 7, 7, 7, 7, 0, 0, 0, 0, 0, 0, 7, 7, 7, 7, 0, 0, 7, 7, 7, 7, 0, 0, 0, 0, 7, 7, 0, 0, 0, 0, 0, 0, 7, 7,
   7, 7, 0, 0, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 0, 0, 7, 7, 7, 0, 0, 7, 7, 0, 0, 7, 7, 0, 0, 7, 7, 7, 7, 7, 7, 7, 0, 0, 0, 7, 7, 0, 0, 7, 7, 7, 0, 0, 7, 7, 7, 0, 0, 7, 7, 0, 0, 7, 7, 7, 0, 0, 7, 7, 7, 7,
   7, 7, 7, 0, 0, 0, 0, 0, 0, 7, 7, 7, 7, 7, 0, 0, 7, 7, 7, 0, 0, 7, 7, 0, 0, 7, 7, 0, 0, 7, 7, 7, 7, 7, 7, 7, 0, 0, 7, 7, 7, 7, 0, 0, 7, 7, 0, 0, 7, 7, 0, 0, 7, 7, 7, 7, 0, 0, 7, 7, 0, 0, 7, 7, 7, 7,
   7, 7, 7, 7, 7, 7, 7, 7, 0, 0, 7, 7, 7, 7, 0, 0, 7, 7, 7, 0, 0, 7, 7, 0, 0, 7, 7, 0, 0, 7, 7, 7, 7, 7, 7, 7, 0, 0, 7, 7, 7, 7, 0, 0, 7, 7, 0, 0, 7, 7, 0, 0, 7, 7, 7, 7, 7, 7, 7, 7, 0, 0, 7, 7, 7, 7,
   7, 7, 7, 7, 7, 7, 7, 7, 7, 0, 0, 7, 7, 7, 0, 0, 7, 7, 7, 0, 0, 7, 7, 0, 0, 7, 7, 0, 0, 7, 7, 7, 7, 7, 7, 7, 0, 0, 7, 7, 7, 7, 0, 0, 7, 7, 0, 0, 7, 7, 0, 0, 7, 7, 7, 7, 7, 7, 7, 7, 0, 0, 7, 7, 7, 7,
   7, 7, 7, 7, 7, 7, 7, 7, 7, 0, 0, 7, 7, 7, 0, 0, 7, 7, 7, 0, 0, 7, 7, 0, 0, 7, 7, 0, 0, 7, 7, 7, 7, 7, 7, 7, 0, 0, 7, 7, 7, 7, 0, 0, 7, 7, 0, 0, 7, 7, 0, 0, 7, 7, 7, 7, 7, 7, 7, 7, 0, 0, 7, 7, 7, 7,
   7, 0, 0, 7, 7, 7, 7, 7, 7, 0, 0, 7, 7, 7, 0, 0, 7, 7, 7, 0, 0, 7, 7, 0, 0, 7, 7, 0, 0, 7, 7, 7, 7, 7, 7, 7, 0, 0, 7, 7, 7, 7, 0, 0, 7, 7, 0, 0, 7, 7, 0, 0, 7, 7, 7, 7, 0, 0, 7, 7, 0, 0, 7, 7, 7, 7,
   7, 7, 0, 0, 7, 7, 7, 7, 0, 0, 7, 7, 7, 7, 0, 0, 7, 7, 7, 0, 0, 7, 7, 0, 0, 7, 7, 0, 0, 7, 7, 7, 7, 7, 7, 7, 0, 0, 0, 7, 7, 0, 0, 7, 7, 7, 0, 0, 7, 7, 7, 0, 0, 7, 7, 0, 0, 7, 7, 7, 0, 0, 7, 7, 7, 7,
   7, 7, 7, 0, 0, 0, 0, 0, 0, 7, 7, 7, 7, 7, 7, 0, 0, 7, 7, 0, 0, 7, 7, 0, 0, 7, 7, 0, 0, 7, 7, 7, 7, 7, 7, 7, 0, 0, 0, 0, 0, 0, 7, 7, 7, 7, 0, 0, 7, 7, 7, 7, 0, 0, 0, 0, 7, 7, 7, 7, 7, 0, 0, 7, 7, 7,
   7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 0, 0, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7,
   7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 0, 0, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7,
   7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 0, 0, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7,
   7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 0, 0, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7,
   7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7,
};

        TG_IMAGE live_but_img = { live_but_pict,
                                  BUT_ICON_DX,
                                  BUT_ICON_DY,
                                  BUT_ICON_DX };

        TG_IMAGE still_but_img = { still_but_pict,
                                   BUT_ICON_DX,
                                   BUT_ICON_DY,
                                   BUT_ICON_DX };

/*............. Function prototypes */
short get_token (
        char   *str,            /* input scanned string                 */
        int    *ipos,           /* input/output position in string      */
        char   *token );        /* output token string                  */
TG_RET fill_box (
        TG_BOX    box,          /* input box                            */
        TG_BOX    win_box,      /* input window box                     */
        TG_COLOR  clr );        /* input color                          */
TG_RET draw_bin_box (
        TG_COORD   x,           /* input x-coord. of top lef box corner */
        TG_COORD   y,           /* input y-coord. of top lef box corner */
        TG_COORD   dx,          /* input box width                      */
        TG_COORD   dy,          /* input box height                     */
        TG_BOX     win_box,     /* input window box                     */
        TG_COLOR   clr,         /* input box color                      */
        int        mode );      /* input drawing mode (SET/XOR)         */
TG_RET update_bin_box ();
TG_RET binarize ();
void conv_ti_colors (
        char   *img_buf,        /* I/O color image buffer               */
        int     dx,             /* input image width                    */
        int     dy );            /* input image height                   */

/* Dummy functions for PC test: */
#ifdef _ZC_31
  void vmode (int mode);
  void tpict ();
#endif


/****** main = Main demo function = 03.2003 *****************************/
#ifdef _TI_CAMERA
  void main (int argc, char *argv)
#else
  void main()
#endif
{
        TG_RET   rc;
        TG_BOX   scr_box;

        char    *win_title = "TG binarization demo";
        TG_COORD win_x,win_y;
        TG_COORD win_dx,win_dy;
        TG_INT16 win_attr;
        TG_INT16 win_depth;
        TG_COORD client_dx,client_dy;

        TG_COORD x,y;
        TG_COORD dx,dy;
        TG_COORD depth;
        TG_COORD msg_x;
        TG_COORD skip_dist;
        TG_BOX   frame_box;

        TG_COLOR tclr = TG_COLOR_BLACK;
//        TG_COLOR tclr = TG_COLOR_BLUE;
//        TG_COLOR tclr = TG_COLOR_WHITE;

/* GUI identifiers */
        TG_INT16 win_id;
        TG_INT16 text_id;
        TG_INT16 frame_id;

        TG_INT16 spin_th_id;
        TG_INT16 spin_clr_id;

        TG_INT16 spin_x_id;
        TG_INT16 spin_y_id;
        TG_INT16 spin_dx_id;
        TG_INT16 spin_dy_id;

        TG_INT16 but_liv_id;    /* "Live picture" button id     */
        TG_INT16 but_still_id;  /* "Still picture" button id    */
        TG_INT16 but_bin_id;    /* "Binarize" button id         */
        TG_INT16 but_exit_id;   /* "Exit" button id             */

        TG_INT16 radb_id;       /* Radio-button menu id         */
        TG_INT16 cbox_id;       /* check-box id                 */

        TG_INT16 gui_id;        /* GUI elemet identifier        */
        TG_EVENT gui_event;     /* current GUI event            */
        TG_INT32    gui_val;

        TG_INT16 font_dx,font_dy;
        int     upd_flg;
        int     spin_step;

        TG_MOUSE ms_event;      /* mouse event          */
        TG_COORD offs_x,offs_y;
        int      begin_drag;
        TG_INT32    bin_on_th_change = 0; /* 1: binarize on threshold change */


//............. Init TG package
        rc = tg_init();
        if(rc != RC_OK)
        {
           print("TG init error = %d\n",rc);
           return;
        }

//............. Set new font
#if CHANGE_FONT
        rc = tg_set_font(FONT_FILE);
//        if(rc != RC_OK) goto done;
#endif

//............. Clear screen
        tg_get_screenbox(scr_box);      /* get screen box */
        rc = tg_fill_box(scr_box,TG_COLOR_CLEAR,TG_MODE_SET);
        TG_ASSERT(rc == RC_OK);
        if(rc != RC_OK) goto done;

//............. Get font size
        tg_get_fontsize(&font_dx,&font_dy);

/*............. Set input mode */
#ifdef _TI_CAMERA
        x = 10;
        y = 10;
        rc = tg_disp_text("Searching mouse...",x,y,
                          TG_COLOR_WHITE,
                          TG_COLOR_BLACK);
        if(rc != RC_OK) goto done;

//        imode = TG_INPUT_MODE_MOUSE;
        rc = tg_set_inputmode(TG_INPUT_MODE_MOUSE);
        TG_ASSERT(rc == RC_OK);
        if(rc != RC_OK) goto done;

        if(tg_mouse_type())     /* mouse found ! */
        {
           rc = tg_disp_text("Mouse detected !",x,y+font_dy + 2,
                             TG_COLOR_WHITE,
                             TG_COLOR_BLACK);
           if(rc != RC_OK) goto done;
        }
        else            /* mouse not found */
        {
           rc = tg_disp_text("Mouse not found, keyboard input enabled",
                             x,y+font_dy+2,
                             TG_COLOR_WHITE,
                             TG_COLOR_BLACK);
           if(rc != RC_OK) goto done;

           rc = tg_set_inputmode(TG_INPUT_MODE_KBD);
           TG_ASSERT(rc == RC_OK);
           if(rc != RC_OK) goto done;
        }
        tg_wait(500);


//        imode = TG_INPUT_MODE_KBD;       /* default input mode = kbd */
//        i = 0;
//        if(get_token(argv,&i,str))
//        {
//           if(!strcmp(str,"-m")) imode = TG_INPUT_MODE_MOUSE;
//        }

#else
        rc = tg_set_inputmode(TG_INPUT_MODE_MOUSE | TG_INPUT_MODE_KBD);
        TG_ASSERT(rc == RC_OK);
        if(rc != RC_OK) goto done;
#endif

//............. Set initial position of bin. box
        bin_x = (tg_box_width(scr_box) - bin_dx) / 2;
        bin_y = (tg_box_height(scr_box) - bin_dy) / 2;

//............. Setup color icon images
        conv_ti_colors(live_but_pict,BUT_ICON_DX,BUT_ICON_DY);
        conv_ti_colors(still_but_pict,BUT_ICON_DX,BUT_ICON_DY);

/*----------------------------------------------------------------------*/
/*                               Open window                            */
/*----------------------------------------------------------------------*/
        win_x = 10;
        win_y = 10;
//        win_dx = 524;
//        win_dy = 148;
        win_dx = 600;
        win_dy = 180;
        win_attr = TG_WIN_CLOSE_BUT;
        win_depth = 2;

        rc = tg_WinOpen(win_title,win_x,win_y,win_dx,win_dy,win_attr,
                        win_depth,
                        &win_id);
        TG_ASSERT(rc == RC_OK);
        if(rc != RC_OK) goto done;

/*............. Get size of client area box */
        rc = tg_WinClientSize(&client_dx,&client_dy);
        TG_ASSERT(rc == RC_OK);
        if(rc != RC_OK) goto done;

/*............. Generate window box */
//        tg_make_box(win_box,win_x,win_y,win_x+win_dx-1,win_y+win_dy-1);
        rc = tg_WinMainBox(win_box);
        if(rc != RC_OK) goto done;

/*----------------------------------------------------------------------*/
/*                          Open window controls                        */
/*----------------------------------------------------------------------*/
/* Coordinates of GUI elements are relative to window client area ! */

        msg_x = 10;
        x = msg_x + 80;
        skip_dist = 10;
        depth = 2;

/*............. Open "Threshold" spin */
        y = 22;
        dx = 100;
        dy = 20;

//        rc = tg_disp_text("Threshold",msg_x,y+(dy-font_dy)/2,tclr,
//                          TG_COLOR_DEF_BGND);
        rc = tg_TextOpen("Threshold",msg_x,y+(dy-font_dy)/2,0,
                          tclr,
                          TG_COLOR_DEF_BGND,
                          &text_id);
        TG_ASSERT(rc == RC_OK);
        if(rc != RC_OK) goto done;

        rc = tg_IntSpinOpen(bin_th,1,1,255,TG_SPIN_EDIT_ON,3,
                            x,y,dx,dy,depth,
                            NULL,               // default colors
                            &spin_th_id);       // GUI element id.
        TG_ASSERT(rc == RC_OK);
        if(rc != RC_OK) goto done;

/*............. Open "Color" spin (toggle) */
        y += dy + skip_dist;

//        rc = tg_disp_text("Color",msg_x,y+(dy-font_dy)/2,tclr,
//                           TG_COLOR_DEF_BGND);
        rc = tg_TextOpen("Color",msg_x,y+(dy-font_dy)/2,0,
                          tclr,
                          TG_COLOR_DEF_BGND,
                          &text_id);
        TG_ASSERT(rc == RC_OK);
        if(rc != RC_OK) goto done;

        rc = tg_ToggleSpinOpen(bin_clr_txt,BIN_COLOR_CNT,bin_clr_num,
                               x,y,dx,dy,depth,
                               NULL,            // default colors
                               &spin_clr_id);   // GUI element id.
        TG_ASSERT(rc == RC_OK);
        if(rc != RC_OK) goto done;

/*............. Open radio-button menu (mark dark/light area) */
        y += dy + 2*skip_dist;
        rc = tg_RadioButOpen(radb_menu_txt,2,mark_area,TG_RADIOBUT_TYPE_VERT,
                             6,msg_x,y,
                             NULL,
                             &radb_id);
        if(rc != RC_OK) goto done;


/*............. Open check-box : "Binarize on threshold change" */
        rc = tg_CheckBoxOpen("Binarize on threshold change",
                              bin_on_th_change,
                              msg_x+152,y,NULL,&cbox_id);
        if(rc != RC_OK) goto done;

//............. Draw "Binarization box" frame
        x += dx + 12;
        y = 10;
        dx = 205;
        dy = 69;
        tg_make_box(frame_box,x,y,x+dx-1,y+dy-1);

//        rc = tg_draw_frame("Binarizarion box",frame_box,NULL,1,0,
//                                TG_ALIGN_CENTER,TG_FRAME_DOWN_UP);
        rc = tg_FrameOpen("Binarizarion box",frame_box,NULL,1,0,
                           TG_ALIGN_CENTER,TG_FRAME_DOWN_UP,&frame_id);
        if(rc != RC_OK) goto done;

//print("w = %d\n",tg_box_width(frame_box));


/*............. Open "x" spin */
        msg_x = frame_box[TG_X1] + 10;
        x = msg_x + 16;
        skip_dist = 10;
        spin_step = 1;
//        depth = 2;

        y = frame_box[TG_Y1] + 12;
        dx = 64;
        dy = 20;

//        rc = tg_disp_text("x",msg_x,y+(dy-font_dy)/2,tclr,TG_COLOR_DEF_BGND);
        rc = tg_TextOpen("x",msg_x,y+(dy-font_dy)/2,0,
                          tclr,
                          TG_COLOR_DEF_BGND,
                          &text_id);
        if(rc != RC_OK) goto done;

        rc = tg_IntSpinOpen(bin_x,spin_step,0,scr_box[TG_X2],
                            TG_SPIN_EDIT_OFF,3,
                            x,y,dx,dy,depth,
                            NULL,               // default colors
                            &spin_x_id);       // GUI element id.
        if(rc != RC_OK) goto done;

/*............. Open "y" spin */
        y += dy + skip_dist;

//        rc = tg_disp_text("y",msg_x,y+(dy-font_dy)/2,tclr,TG_COLOR_DEF_BGND);
        rc = tg_TextOpen("y",msg_x,y+(dy-font_dy)/2,0,
                          tclr,
                          TG_COLOR_DEF_BGND,
                          &text_id);
        if(rc != RC_OK) goto done;

        rc = tg_IntSpinOpen(bin_y,spin_step,0,scr_box[TG_Y2],
                            TG_SPIN_EDIT_OFF,3,
                            x,y,dx,dy,depth,
                            NULL,               // default colors
                            &spin_y_id);       // GUI element id.
        if(rc != RC_OK) goto done;


/*............. Open "dx" spin */
        msg_x = x + dx + 16;
        x = msg_x + 24;
        y = frame_box[TG_Y1] + 12;
        spin_step = 2;

//        rc = tg_disp_text("dx",msg_x,y+(dy-font_dy)/2,tclr,TG_COLOR_DEF_BGND);
        rc = tg_TextOpen("dx",msg_x,y+(dy-font_dy)/2,0,
                          tclr,
                          TG_COLOR_DEF_BGND,
                          &text_id);
        if(rc != RC_OK) goto done;

        rc = tg_IntSpinOpen(bin_dx,spin_step,8,tg_box_width(scr_box),
                            TG_SPIN_EDIT_ON,3,
                            x,y,dx,dy,depth,
                            NULL,               // default colors
                            &spin_dx_id);       // GUI element id.
        if(rc != RC_OK) goto done;


/*............. Open "dy" spin */
        y += dy + skip_dist;

//        rc = tg_disp_text("dy",msg_x,y+(dy-font_dy)/2,tclr,TG_COLOR_DEF_BGND);
        rc = tg_TextOpen("dy",msg_x,y+(dy-font_dy)/2,0,
                          tclr,
                          TG_COLOR_DEF_BGND,
                          &text_id);
        if(rc != RC_OK) goto done;

        rc = tg_IntSpinOpen(bin_dy,spin_step,8,tg_box_height(scr_box),
                            TG_SPIN_EDIT_ON,3,
                            x,y,dx,dy,depth,
                            NULL,               // default colors
                            &spin_dy_id);       // GUI element id.
        if(rc != RC_OK) goto done;


/*............. Open "Live pict" button */
        dx = BUT_ICON_DX + 8 + 2*depth;
        dy = BUT_ICON_DY + 8 + 2*depth;
//        dx = max(BUT_ICON_DX,BUT_ICON_DY) + 8 + 2*depth;
//        dy = dx;
        skip_dist = 10;
//        x = (client_dx - 4*dx - 3*skip_dist) / 2;
        x = frame_box[TG_X2] + 12;
        y = 10;

        rc = tg_IconButOpen(&live_but_img,
                            TG_BUT_TYPE_CLR_ICON,
                            x,y,dx,dy,depth,
                            TG_ALIGN_CENTER,
                            NULL,
                            &but_liv_id);


/*............. Open "Still pict" button */
        x += dx + skip_dist;
        rc = tg_IconButOpen(&still_but_img,
                            TG_BUT_TYPE_CLR_ICON,
                            x,y,dx,dy,depth,
                            TG_ALIGN_CENTER,
                            NULL,
                            &but_still_id);


/*............. Open "Binarize" button */
//        x += dx + skip_dist;
        x = frame_box[TG_X2] + 12;
        y += dy + skip_dist;
        dx = 2*dx + skip_dist;
        dy = 25;
        rc = tg_TxtButOpen("Binarize",x,y,dx,dy,depth,
                         TG_ALIGN_CENTER,
                         NULL,
                         &but_bin_id);
        if(rc != RC_OK) goto done;

/*............. Open "Exit" button */
        y += dy + skip_dist;
        rc = tg_TxtButOpen("Exit (F4)",x,y,dx,dy,depth,
                         TG_ALIGN_CENTER,
                         NULL,
                         &but_exit_id);
        if(rc != RC_OK) goto done;


/*----------------------------------------------------------------------*/
/*                          Draw binarization box                       */
/*----------------------------------------------------------------------*/
        rc = update_bin_box();
        if(rc != RC_OK) goto done;

/*----------------------------------------------------------------------*/
/*                          Process GUI events                          */
/*----------------------------------------------------------------------*/

        for(;;)         /* get event loop */
        {
           rc = tg_GetEvent(&gui_id,&gui_event);
           if(rc != RC_OK) goto done;

/*............. Get window box */
           x = win_box[TG_X1];          /* save old position */
           y = win_box[TG_Y1];
           rc = tg_WinMainBox(win_box);
           if(rc != RC_OK) goto done;


/*............. Window is moved: clear screen box */
           if(x != win_box[TG_X1] || y != win_box[TG_Y1])
           {
              rc = fill_box(scr_box,win_box,TG_COLOR_CLEAR);
              if(rc != RC_OK) goto done;

              bin_fill = 0;
              rc = draw_bin_box(bin_x,bin_y,bin_dx,bin_dy,win_box,
                          bin_box_clr,TG_MODE_XOR);
              if(rc != RC_OK) goto done;
           }

/*............. Window events */
//           if(gui_id == win_id && gui_event == TG_WIN_EVENT_CLOSE)
           if(gui_event == TG_WIN_EVENT_CLOSE)
               break;

/*                         --------------------                         */
/*.........................     Spin events    .........................*/
/*                         --------------------                         */
           if(gui_event == TG_SPIN_EVENT_CHANGE)
           {
/*............. Get spin value */
              rc = tg_GetIntData(gui_id,&gui_val);
              if(rc != RC_OK) goto done;

/*............. Process respective spin event */
              upd_flg = 0;

              if(gui_id == spin_th_id)          /* Bin. threshold */
              {
                 bin_th = gui_val;
                 if(bin_on_th_change)
                 {
                    rc = binarize();
                    if(rc != RC_OK) goto done;
                 }
              }

              if(gui_id == spin_clr_id)         /* Bin. color */
              {
                 bin_clr_num = gui_val;
              }

              if(gui_id == spin_x_id)           /* "x" spin */
              {
                 bin_x = gui_val;
                 upd_flg = 1;
              }
              if(gui_id == spin_y_id)           /* "y" spin */
              {
                 bin_y = gui_val;
                 upd_flg = 1;
              }
              if(gui_id == spin_dx_id)          /* "dx" spin */
              {
                 bin_dx = gui_val;
                 upd_flg = 1;
              }
              if(gui_id == spin_dy_id)          /* "dy" spin */
              {
                 bin_dy = gui_val;
                 upd_flg = 1;
              }

/*............. Update bin. box */
              if(upd_flg)
              {
                 rc = update_bin_box();
                 if(rc != RC_OK) goto done;
              }

           } /* if(gui_event == TG_SPIN_EVENT_CHANGE) */


/*                        ---------------------                         */
/*........................ Radio-button events .........................*/
/*                        ---------------------                         */
           if(gui_event == TG_RADIOBUT_EVENT_CHANGE)
           {

/*............. Get menu value */
              rc = tg_GetIntData(gui_id,&mark_area);
              if(rc != RC_OK) goto done;

              rc = binarize();
              if(rc != RC_OK) goto done;

           } /* if(gui_event == TG_RADIOBUT_EVENT_CHANGE) */

/*                        ---------------------                         */
/*........................   Check-box events  .........................*/
/*                        ---------------------                         */
           if(gui_id == cbox_id &&
              gui_event == TG_CHECKBOX_EVENT_CHANGE)
           {
              rc = tg_GetIntData(gui_id,&bin_on_th_change);
              if(rc != RC_OK) goto done;
           }

/*                         --------------------                         */
/*.........................    Button events   .........................*/
/*                         --------------------                         */
           if(gui_event == TG_BUT_EVENT_CLICK)
           {

/*............. "Live pict" button pressed */
              if(gui_id == but_liv_id)
              {
                 vmode(4);
                 rc = update_bin_box();         /* update bin. box */
                 if(rc != RC_OK) goto done;
                 continue;
              }

/*............. "Still pict" button pressed */
              if(gui_id == but_still_id)
              {
                 vmode(5);
//                 tpict();
                 rc = update_bin_box();         /* update bin. box */
                 if(rc != RC_OK) goto done;
                 continue;
              }

/*............. "Binarize" button pressed */
              if(gui_id == but_bin_id)
              {
                 rc = binarize();
                 if(rc != RC_OK) goto done;
                 continue;
              }

/*............. "Exit" button pressed */
             if(gui_id == but_exit_id)
                 break;

           } /* if(gui_event == TG_BUT_EVENT_CLICK) */

/*............. Drag binarization box by left mouse button */
              begin_drag = 0;
              for(;;)                   /* Drag loop */
              {
                 tg_GetMouseEvent(&ms_event);
//                 x_old = x;
//                 y_old = y;
                 x = ms_event.x;
                 y = ms_event.y;
                 if((ms_event.stat & TG_MOUSE_BUT_LEFT) &&
                     tg_point_inbox(bin_box,x,y) &&
                     !tg_point_inbox(win_box,x,y))
                 {
                    if(begin_drag == 0)
                    {
                       offs_x = x - bin_x;
                       offs_y = y - bin_y;

                       rc = update_bin_box();    /* clear filled box */
                       if(rc != RC_OK) goto done;

                       begin_drag = 1;
                    }
                    bin_x = x - offs_x;
                    bin_y = y - offs_y;
                    if(bin_x < scr_box[TG_X1]) bin_x = scr_box[TG_X1];
                    if(bin_x > scr_box[TG_X2]) bin_x = scr_box[TG_X2];
                    if(bin_y < scr_box[TG_Y1]) bin_y = scr_box[TG_Y1];
                    if(bin_y > scr_box[TG_Y2]) bin_y = scr_box[TG_Y2];
                    if(bin_x != bin_box[TG_X1] || bin_y != bin_box[TG_Y1])
                    {
                       rc = update_bin_box();
                       if(rc != RC_OK) goto done;
                    }

                    /* Pump next mouse event */
                    rc = tg_PumpMouseEvent();
                    if(rc != RC_OK) goto done;

                 }
                 else    /* end of drag */
                 {
                    if(begin_drag)      /* update x/y spins */
                    {
                       rc = tg_SetGuiData(spin_x_id,&bin_x);
                       if(rc != RC_OK) goto done;
                       rc = tg_SetGuiData(spin_y_id,&bin_y);
                      if(rc != RC_OK) goto done;
                    }
                    break;
                 }

              } /* for(;;) : drag loop */

        } /* for(;;) : get event loop */

//............. Close window
        rc = tg_WinClose();
        if(rc != RC_OK) goto done;

        rc = tg_fill_box(scr_box,TG_COLOR_CLEAR,TG_MODE_SET);
        if(rc != RC_OK) goto done;

/*............. Exit */
done:
        tg_exit();
        if(rc != RC_OK)
           print("RC = %d\n",rc);

        return;
}

#ifdef _TI_CAMERA       /* [[[ */
/****** get_token = Get Token from String = 03.2003 *********************/
/*----------------------------------------------------------------------*/
/**
* Get token from string.
*
* The function extracts a token substring from "str", starting from
* position "ipos", stores the substring into output "token" string
* and updates "ipos" for the next extraction. The tokens are
* separated by one or more blanks.
*
* <p> Note: The output token buffer is not checked for overflow.
*
* @param  str    Input string to get token from
* @param  ipos   Input/output start search position of token in str
*                (*ipos >= 0)
* @param  token  Output token string
*
* @return
*      >0 : Length of the token substring
*       0 : No more tokens in "str"
*
* @version
*       Created:    Plamen Kozhuharov, 11.03.2003
* <br>  Modified:
*/
/*----------------------------------------------------------------------*/

short get_token (
        char   *str,            /* input scanned string                 */
        int    *ipos,           /* input/output position in string      */
        char   *token )         /* output token string                  */
{

        int     i1,i2;
        int     len;
        char   *sptr;
        char    blank = ' ';

/*............. check input string */
        len = strlen(str);
        if(len == 0 || *ipos >= len) return(0);

/*............. seek first non-blank char at position i1 */
        i1 = *ipos;
        sptr = str + i1;
        while(*sptr++ == blank)
        {
          i1++;
          if(i1 >= len) return(0);
        }

/*............. find next blank (end of token) */
        i2 = i1 + 1;
        sptr = str + i2;
        while(*sptr++ != blank)
        {
          i2++;
          if(i2 >= len) break;
        }

/*............. get token from str */
        *ipos = i2 + 1;
        len = i2 - i1;
        memcpy(token,str+i1,len);
        token[len] = '\0';
        return(len);

}
#endif  /* ]]] */

/****** fill_box = Fill Box with Color = 03.2003 ************************/
/*----------------------------------------------------------------------*/
/**
* Fill box with color.
*
* The function fills input box "box" with color, but skips filling the
* window box "win_box". The two boxes have absolute coordinates.
*
* @param  box      Input box, filled with color
* @param  win_box  Input window box, skipped from filling
* @param  clr      Input fill color
*
* @return
*       RC_OK = OK
* <br>  Other = Returned by the called functions
*
* @version
*       Created:    Plamen Kozhuharov, 21.03.2003
* <br>  Modified:
*/
/*----------------------------------------------------------------------*/

TG_RET fill_box (
        TG_BOX    box,          /* input box                            */
        TG_BOX    win_box,      /* input window box                     */
        TG_COLOR  clr )         /* input color                          */
{
        TG_RET    rc;
        TG_COORD  org_x,org_y;
        TG_BOX    fbox;

/*............. Setup */
        tg_get_draworg(&org_x,&org_y);  /* save current drawing origin */
        tg_set_draworg(0,0);            /* set absolute drawing origin */
//         tg_hide_cursor();

/*............. No common area between box and win_box */
        if(tg_clip_box(box,win_box,fbox))
        {
           rc = tg_fill_box(box,clr,TG_MODE_SET);
//           if(rc != RC_OK) goto done;
           goto done;
        }

/*............. Fill "box" area above "win_box" */
        if(box[TG_Y1] < win_box[TG_Y1])
        {
           tg_make_box(fbox,
                       box[TG_X1], box[TG_Y1],
                       box[TG_X2], win_box[TG_Y1]-1);
           rc = tg_fill_box(fbox,clr,TG_MODE_SET);
           if(rc != RC_OK) goto done;
        }

/*............. Fill "box" area below "win_box" */
        if(box[TG_Y2] > win_box[TG_Y2])
        {
           tg_make_box(fbox,
                       box[TG_X1], win_box[TG_Y2]+1,
                       box[TG_X2], box[TG_Y2] );
           rc = tg_fill_box(fbox,clr,TG_MODE_SET);
           if(rc != RC_OK) goto done;
        }

/*............. Fill "box" area left from "win_box" */
        if(box[TG_X1] < win_box[TG_X1])
        {
           tg_make_box(fbox,
                       box[TG_X1],       max(box[TG_Y1],win_box[TG_Y1]),
                       win_box[TG_X1]-1, min(box[TG_Y2],win_box[TG_Y2]));
           rc = tg_fill_box(fbox,clr,TG_MODE_SET);
           if(rc != RC_OK) goto done;
        }

/*............. Fill "box" area right from "win_box" */
        if(box[TG_X2] > win_box[TG_X2])
        {
           tg_make_box(fbox,
                       win_box[TG_X2]+1, max(box[TG_Y1],win_box[TG_Y1]),
                       box[TG_X2],       min(box[TG_Y2],win_box[TG_Y2]));
           rc = tg_fill_box(fbox,clr,TG_MODE_SET);
           if(rc != RC_OK) goto done;
        }

/*............. Exit */
done:
//        rc = tg_show_cursor();
//        if(rc != RC_OK) return rc;

        tg_set_draworg(org_x,org_y);  /* restore drawing origin */
        return rc;

}

/****** draw_bin_box = Draw Binarization Box = 03.2003 ******************/
/*----------------------------------------------------------------------*/
/**
* Draw binarization box.
*
* The function draws binarization box (absolute coordinates). The box is
* truncated to fit into screen. Box dots, which lie inside "win_box",
* are skipped from drawing.
*
* @param  x        Input x-coord. of top lef box corner (absolute)
* @param  y        Input y-coord. of top lef box corner (absolute)
* @param  dx       Input box width in pixels
* @param  dy       Input box height in pixels
* @param  win_box  Input window box, skipped from drawing
*                  (absolute coordinates)
* @param  clr      Input box color
* @param  mode     Input drawing mode:
* <br>               TG_MODE_SET = draw in SET mode
* <br>               TG_MODE_XOR = draw in XOR mode
*
* @return
*       RC_OK = OK
* <br>  Other = Returned by the called functions
*
* @version
*       Created:    Plamen Kozhuharov, 12.03.2003
* <br>  Modified:
*/
/*----------------------------------------------------------------------*/

TG_RET draw_bin_box (
        TG_COORD   x,           /* input x-coord. of top lef box corner */
        TG_COORD   y,           /* input y-coord. of top lef box corner */
        TG_COORD   dx,          /* input box width                      */
        TG_COORD   dy,          /* input box height                     */
        TG_BOX     win_box,     /* input window box                     */
        TG_COLOR   clr,         /* input box color                      */
        int        mode )       /* input drawing mode (SET/XOR)         */
{

        TG_RET    rc;
        TG_COORD  org_x,org_y;
        int       clip_mode;
        TG_BOX    clip_box;
        TG_BOX    scr_box;
        TG_BOX    bin_box1;

/*............. Set absolute drawing origin */
        tg_get_draworg(&org_x,&org_y);  /* save current drawing origin */
        tg_set_draworg(0,0);

/*............. Set inverse clipping mode (draw outside win_box) */
        clip_mode = tg_get_clipmode();  /* save current clipping mode */
        tg_get_clipbox(clip_box);       /* save current clipping box  */

        tg_set_clipmode(TG_CLIP_INV);
        tg_set_clipbox(win_box);

/*............. Truncate binar. box inside screen */
        tg_get_screenbox(scr_box);      /* get screen box */
        tg_make_box(bin_box,x,y,x+dx-1,y+dy-1);
        if(bin_box[TG_X2] > scr_box[TG_X2]) bin_box[TG_X2] = scr_box[TG_X2];
        if(bin_box[TG_Y2] > scr_box[TG_Y2]) bin_box[TG_Y2] = scr_box[TG_Y2];

/*............. Clear binarization box (if filled with color) */
        if(bin_fill)
        {
           /* skip border lines */
           tg_make_box(bin_box1,
                       bin_box[TG_X1]+1,bin_box[TG_Y1]+1,
                       bin_box[TG_X2]-1,bin_box[TG_Y2]-1);

           if(bin_box1[TG_X1] < bin_box1[TG_X2] &&
              bin_box1[TG_Y1] < bin_box1[TG_Y2])
           {
              rc = fill_box(bin_box1,win_box,TG_COLOR_CLEAR);
              if(rc != RC_OK) goto done;
           }

           bin_fill = 0;
        }

/*............. Draw binarization box */
        rc = tg_draw_box(bin_box,TG_LINE_SOLID,clr,mode);
        if(rc != RC_OK) goto done;

/*............. Exit */
done:
        tg_set_clipbox(clip_box);     /* restore clipping box   */
        tg_set_clipmode(clip_mode);   /* restore clipping mode  */
        tg_set_draworg(org_x,org_y);  /* restore drawing origin */
        return rc;

}

/****** update_bin_box = Update Binarization Box = 03.2003 **************/
/*----------------------------------------------------------------------*/
/**
* Update binarization box.
*
* The function updates (redraws) binarization box when some box parameter
* has been changed.
*
* @param
*
* @return
*       RC_OK = OK
* <br>  Other = Returned by the called functions
*
* @version
*       Created:    Plamen Kozhuharov, 12.03.2003
* <br>  Modified:
*/
/*----------------------------------------------------------------------*/

TG_RET update_bin_box ()
{
        TG_RET  rc;

/* Old box parameters */
        static TG_INT32 bin_x1 = -1;
        static TG_INT32 bin_y1;
        static TG_INT32 bin_dx1;
        static TG_INT32 bin_dy1;


/*............. Erase old bin. box */
        if(bin_x1 >= 0)         /* valid old coordinates */
        {
           rc = draw_bin_box(bin_x1,bin_y1,bin_dx1,bin_dy1,win_box,
                             bin_box_clr,TG_MODE_XOR);
           if(rc != RC_OK) return rc;

//           rc = clear_bin_box();
//           if(rc != RC_OK) return rc;
        }

/*............. Draw new bin. box */
        rc = draw_bin_box(bin_x,bin_y,bin_dx,bin_dy,win_box,
                          bin_box_clr,TG_MODE_XOR);
        if(rc != RC_OK) return rc;

/*............. Save parameters of current bin. box */
        bin_x1  = bin_x;
        bin_y1  = bin_y;
        bin_dx1 = bin_dx;
        bin_dy1 = bin_dy;

        return RC_OK;

}

/****** binarize = Binarize = 03.2003 ***********************************/
/*----------------------------------------------------------------------*/
/**
* Binarize.
*
* The function performs binarization of the screen box, specified by
* the global box bin_box, with threshold bin_th.
*
* @param
*
* @return
*       RC_OK = OK
* <br>  Other = Returned by the called functions
*
* @version
*       Created:    Plamen Kozhuharov, 12.03.2003
* <br>  Modified:
*/
/*----------------------------------------------------------------------*/

TG_RET binarize ()
{

/* DDD */
#define DUMP_IMAGE_DATA  0

        TG_RET    rc = RC_OK;
        TG_COORD  org_x,org_y;
        TG_COORD  x,y;
        TG_COORD  img_dx,img_dy;
        TG_COLOR  bclr;
        TG_COLOR  clr;
        TG_UINT8    *ptr;
        TG_UINT8    *dptr;

/*............. Set absolute drawing origin */
        tg_get_draworg(&org_x,&org_y);  /* save current drawing origin */
        tg_set_draworg(0,0);

/*............. Get image dimensions */
#ifdef _TI_CAMERA
        img_dx = getvar(HWIDTH);
        img_dy = getvar(VWIDTH);
#else
{
        TG_BOX    scr_box;

        tg_get_screenbox(scr_box);      /* get screen box */
        img_dx = scr_box[TG_X2] + 1;
        img_dy = scr_box[TG_Y2] + 1;
}
#endif

/*............. Do a binarization */
        bclr = bin_clrs[bin_clr_num];
//        tg_hide_cursor();    /* restore screen area below cursor */

/*
x = bin_box[TG_X1];
y = bin_box[TG_Y1];
print("x,y=%d,%d\n",(int)x,(int)y);
print("CAPT_START=%u: ptr=%u\n",
              getvar(CAPT_START),
              getvar(CAPT_START) + y*TG_LPITCH + x);
*/

        for(y=bin_box[TG_Y1]+1; y<=bin_box[TG_Y2]-1; y++)
        {
           if(y >= img_dy) break;

#if DUMP_IMAGE_DATA
  print("%03d: ",(int)y);
#endif

#ifdef _TI_CAMERA
           ptr = (TG_UINT8 *)(getvar(CAPT_START) + y*TG_LPITCH + bin_box[TG_X1]);
           dptr = (TG_UINT8 *)(getvar(OVLY_START) + y*TG_LPITCH + bin_box[TG_X1]);
#endif
           for(x=bin_box[TG_X1]+1; x<=bin_box[TG_X2]-1; x++)
           {

              /* outside image ? */
              if(x >= img_dx) break;

#ifdef _TI_CAMERA
              ptr++;
              dptr++;

#if DUMP_IMAGE_DATA
  print("%d ",(int)*ptr);
#endif

#endif

              /* inside window box ? */
              if(x >= win_box[TG_X1] && x <= win_box[TG_X2] &&
                 y >= win_box[TG_Y1] && y <= win_box[TG_Y2]) continue;

#ifdef _TI_CAMERA
              if(mark_area == 0)        /* mark dark area */
              {
                 clr = (*ptr <= bin_th) ? bclr : TG_COLOR_CLEAR;
              }
              else                      /* mark light area */
              {
                 clr = (*ptr >= bin_th) ? bclr : TG_COLOR_CLEAR;
              }

              *dptr = tg_encode_ticlr(clr);
#else
              rc = tg_draw_dot(x,y,bclr,TG_MODE_SET);
              if(rc != RC_OK) break;
#endif
           }

#if DUMP_IMAGE_DATA
  pstr("\n");
#endif
        }

        bin_fill = 1;

//        rc = tg_show_cursor();       /* save cursor area, show cursor */
//        if(rc != RC_OK) return rc;

/*............. Exit */
        tg_set_draworg(org_x,org_y);  /* restore drawing origin */
        return rc;

}


/****** conv_ti_colors = Convert TI Colors = 03.2003 ********************/
void conv_ti_colors (
        char   *img_buf,        /* I/O color image buffer               */
        int     dx,             /* input image width                    */
        int     dy )            /* input image height                   */
{
#ifdef _TI_CAMERA
        int  i;
        for(i=0; i<dx*dy; i++)
        {
           if(*img_buf == 0) *img_buf = TG_COLOR_BLACK;
           *img_buf = tg_encode_ticlr(*img_buf);
           img_buf++;
        }
#endif
        return;
}

/* Dummy functions for PC test: */
#ifdef _ZC_31

/****** vmode = Set Video Modes = 03.2003 *******************************/
void vmode (int mode)
{
  return;
}

/****** tpict = Take Picture = 03.2003 **********************************/
void tpict ()
{
  return;
}

#endif
